// ../common/src/ast.ts
import { babelParse as _babelParse, walkIdentifiers } from "@vue/compiler-sfc";
function babelParse(code, lang) {
  const plugins = [];
  if (lang === "ts" || lang === "tsx")
    plugins.push("typescript");
  if (lang === "jsx" || lang === "tsx")
    plugins.push("jsx");
  const { program } = _babelParse(code, {
    sourceType: "module",
    plugins
  });
  return program;
}
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : test(node.callee.name)));
}

// ../common/src/constants.ts
var DEFINE_PROPS = "defineProps";
var DEFINE_EMITS = "defineEmits";
var DEFINE_OPTIONS = "defineOptions";
var DEFINE_MODEL = "defineModel";
var REPO_ISSUE_URL = "https://github.com/sxzz/unplugin-vue-macros/issues";

// ../common/src/context.ts
import { MagicString } from "vue/compiler-sfc";

// ../common/src/vue.ts
import { compileScript, parse } from "@vue/compiler-sfc";
var parseSFC = (code, id) => {
  var _a;
  const { descriptor } = parse(code, {
    filename: id
  });
  const lang = (_a = descriptor.script || descriptor.scriptSetup) == null ? void 0 : _a.lang;
  let scriptCompiled;
  return {
    ...descriptor,
    lang,
    get scriptCompiled() {
      if (scriptCompiled)
        return scriptCompiled;
      return scriptCompiled = compileScript(descriptor, {
        id
      });
    }
  };
};
var addToScript = (ctx) => {
  const { scriptCode } = ctx;
  if (scriptCode.prepend.length + scriptCode.append.length === 0) {
    return;
  }
  const { sfc, s } = ctx;
  const { script, scriptSetup, lang } = sfc;
  if (script) {
    if (scriptCode.prepend) {
      s.appendRight(script.loc.start.offset, scriptCode.prepend);
      script.content = scriptCode.prepend + script.content;
    }
    if (scriptCode.append) {
      s.appendRight(script.loc.end.offset, scriptCode.append);
      script.content = script.content + scriptCode.append;
    }
  } else {
    const attrs = lang ? ` lang="${lang}"` : "";
    const content = `${scriptCode.prepend}
${scriptCode.append}`;
    s.prependLeft(0, `<script${attrs}>${content}<\/script>
`);
    sfc.script = {
      type: "script",
      content,
      attrs: (scriptSetup == null ? void 0 : scriptSetup.attrs) || {},
      loc: void 0
    };
  }
};

// ../common/src/context.ts
var initContext = (code, id) => {
  let s;
  let sfc;
  const ctx = {
    code,
    id,
    scriptCode: {
      prepend: "",
      append: ""
    },
    get s() {
      return s || (s = new MagicString(code));
    },
    set s(_s) {
      s = _s;
    },
    get sfc() {
      return sfc || (sfc = parseSFC(code, id));
    },
    set sfc(_sfc) {
      sfc = _sfc;
    }
  };
  return {
    ctx,
    getMagicString() {
      return s;
    },
    getSFC() {
      return sfc;
    }
  };
};
var finalizeContext = (ctx) => {
  addToScript(ctx);
};

// ../common/src/unplugin.ts
var getTransformResult = (s, id) => {
  if (s && s.original !== s.toString()) {
    return {
      code: s.toString(),
      get map() {
        return s.generateMap({
          source: id,
          includeContent: true
        });
      }
    };
  }
};

export {
  babelParse,
  isCallOf,
  DEFINE_PROPS,
  DEFINE_EMITS,
  DEFINE_OPTIONS,
  DEFINE_MODEL,
  REPO_ISSUE_URL,
  initContext,
  finalizeContext,
  getTransformResult
};
