"use strict";Object.defineProperty(exports, "__esModule", {value: true});

var _chunkDCRYUYB2js = require('./chunk-DCRYUYB2.js');

// src/hoist-static/transfrom.ts
var transformHoistStatic = (ctx) => {
  function moveToScript(decl, prefix = "") {
    const text = s.slice(startOffset + decl.start, startOffset + decl.end);
    scriptCode.prepend += `
${prefix}${text}`;
    removeScriptSetup(decl.start, decl.end);
  }
  function removeScriptSetup(start, end) {
    s.remove(startOffset + start, startOffset + end);
    const { content } = scriptSetup;
    scriptSetup.content = `${content.slice(0, start)}${" ".repeat(
      end - start
    )}${content.slice(end)}`;
  }
  const { sfc, s, scriptCode } = ctx;
  const { scriptSetup, lang } = sfc;
  if (!scriptSetup)
    return;
  const startOffset = scriptSetup.loc.start.offset;
  const program = _chunkDCRYUYB2js.babelParse.call(void 0, scriptSetup.loc.source, lang);
  for (const stmt of program.body) {
    if (stmt.type === "VariableDeclaration" && stmt.kind === "const") {
      const decls = stmt.declarations;
      let count = 0;
      for (const [i, decl] of decls.entries()) {
        if (!decl.init || !isLiteral(decl.init))
          continue;
        count++;
        moveToScript(decl, "const ");
        if (decls.length > 1) {
          const isLast = i === decls.length - 1;
          const start = isLast ? decls[i - 1].end : decl.end;
          const end = isLast ? decl.start : decls[i + 1].start;
          s.remove(startOffset + start, startOffset + end);
        }
      }
      if (count === decls.length) {
        removeScriptSetup(stmt.start, stmt.end);
      }
    } else if (stmt.type === "TSEnumDeclaration") {
      const isAllConstant = stmt.members.every(
        (member) => !member.initializer || isLiteral(member.initializer)
      );
      if (!isAllConstant)
        continue;
      moveToScript(stmt);
    }
  }
};
var isLiteral = (node) => {
  var _a;
  if ((_a = node.leadingComments) == null ? void 0 : _a.some(
    (comment) => comment.value.trim() === "hoist-static"
  ))
    return true;
  switch (node.type) {
    case "UnaryExpression":
      return isLiteral(node.argument);
    case "LogicalExpression":
    case "BinaryExpression":
      return isLiteral(node.left) && isLiteral(node.right);
    case "ConditionalExpression":
      return isLiteral(node.test) ? isLiteral(node.consequent) : isLiteral(node.alternate);
    case "SequenceExpression":
    case "TemplateLiteral":
      return node.expressions.every((expr) => isLiteral(expr));
    case "ArrayExpression":
      return node.elements.every((element) => element && isLiteral(element));
    case "ObjectExpression":
      return node.properties.every(
        (prop) => prop.type === "ObjectProperty" && (isLiteral(prop.key) || !prop.computed) && isLiteral(prop.value)
      );
    case "ParenthesizedExpression":
    case "TSNonNullExpression":
    case "TSAsExpression":
    case "TSTypeAssertion":
      return isLiteral(node.expression);
  }
  if (isLiteralType(node))
    return true;
  return false;
};
var isLiteralType = (node) => node.type.endsWith("Literal");



exports.transformHoistStatic = transformHoistStatic;
